@@ -27,9 +27,7 @@
 #include <tchar.h>
 #endif
 
-#ifndef UNDER_CE
 #include <signal.h>
-#endif
 
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -143,7 +141,7 @@ static void memory_tracking_init(void)
 /*
 ** curl tool main function.
 */
-#if defined(_UNICODE) && !defined(UNDER_CE)
+#ifdef _UNICODE
 #if defined(__GNUC__) || defined(__clang__)
 /* GCC does not know about wmain() */
 #pragma GCC diagnostic push
@@ -159,18 +157,17 @@ int main(int argc, char *argv[])
 
   tool_init_stderr();
 
-#if defined(_WIN32) && !defined(UNDER_CE)
+#ifdef _WIN32
   /* Undocumented diagnostic option to list the full paths of all loaded
      modules. This is purposely pre-init. */
   if(argc == 2 && !_tcscmp(argv[1], _T("--dump-module-paths"))) {
     struct curl_slist *item, *head = GetLoadedModulePaths();
     for(item = head; item; item = item->next)
-      printf("%s\n", item->data);
+      curl_mprintf("%s\n", item->data);
     curl_slist_free_all(head);
     return head ? 0 : 1;
   }
-#endif
-#ifdef _WIN32
+
   /* win32_init must be called before other init routines. */
   result = win32_init();
   if(result) {
@@ -214,7 +211,7 @@ int main(int argc, char *argv[])
 #endif
 }
 
-#if defined(_UNICODE) && !defined(UNDER_CE)
+#ifdef _UNICODE
 #if defined(__GNUC__) || defined(__clang__)
 #pragma GCC diagnostic pop
 #endif
