@@ -32,7 +32,6 @@
 #include "unit1615.c"
 #include "unit1616.c"
 #include "unit1620.c"
-#include "unit1636.c"
 #include "unit1650.c"
 #include "unit1651.c"
 #include "unit1652.c"
@@ -60,7 +59,6 @@
 #include "unit3212.c"
 #include "unit3213.c"
 #include "unit3214.c"
-#include "unit3216.c"
 
 const struct entry_s s_entries[] = {
   {"unit1300", test_unit1300},
@@ -93,7 +91,6 @@ const struct entry_s s_entries[] = {
   {"unit1615", test_unit1615},
   {"unit1616", test_unit1616},
   {"unit1620", test_unit1620},
-  {"unit1636", test_unit1636},
   {"unit1650", test_unit1650},
   {"unit1651", test_unit1651},
   {"unit1652", test_unit1652},
@@ -121,7 +118,6 @@ const struct entry_s s_entries[] = {
   {"unit3212", test_unit3212},
   {"unit3213", test_unit3213},
   {"unit3214", test_unit3214},
-  {"unit3216", test_unit3216},
   {NULL, NULL}
 };
 
